var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

import React, { Component } from 'react';
import PropTypes from 'prop-types';
import { ContentState, Editor, EditorState, convertFromRaw } from 'draft-js';
import _debounce from 'lodash.debounce';
import TypeLine from './TypeLine';
import FontFaceObserver from 'fontfaceobserver';

var TypeTester = function (_Component) {
  _inherits(TypeTester, _Component);

  function TypeTester(props) {
    _classCallCheck(this, TypeTester);

    var _this = _possibleConstructorReturn(this, _Component.call(this, props));

    var lines = props.value.split(/[\n\r]/g).map(function (line, index) {
      return {
        text: line,
        key: 'TypeTester_' + line.substring(0, 10) + '_' + index,
        type: 'unstyled',
        entityRanges: []
      };
    });

    var emptyContentState = convertFromRaw({
      entityMap: {},
      blocks: lines
    });

    _this.state = {
      editorState: EditorState.createWithContent(emptyContentState),
      valueOriginal: _this.props.value,
      valueLabel: _this.props.value.substr(0, 30),
      fontsLoaded: false,
      scaled: false,
      caret: _this.props.caret !== false
    };

    _this.handleEditorParentRef = function (ref) {
      return _this.editorParentRef = ref;
    };
    _this.handleEditorRef = function (ref) {
      return _this.editorRef = ref;
    };

    _this.dimensions = _this.dimensions.bind(_this);
    _this.handleOnClickParent = _this.handleOnClickParent.bind(_this);
    _this.handleOnFocus = _this.handleOnFocus.bind(_this);
    _this.handleOnChange = _this.handleOnChange.bind(_this);
    _this.handleAnyChange = _this.handleAnyChange.bind(_this);
    _this.resizeAll = _this.resizeAll.bind(_this);
    _this.initFontCheck = _this.initFontCheck.bind(_this);

    _this.handleResize = _this.handleResize.bind(_this);
    _this.handleResize = _debounce(_this.handleResize, 100);
    return _this;
  }

  TypeTester.prototype.handleOnClickParent = function handleOnClickParent(e) {
    if (this.editorRef) {
      this.setState({ caret: false });
      this.editorRef.focus();
    }
  };

  TypeTester.prototype.handleOnFocus = function handleOnFocus(e) {
    var props = this.props;
    this.setState({ caret: false });

    if (props.onClick) {
      if (e && e.target && !e.target.value) {
        e.target.value = this.state.valueLabel;
      }
      return props.onClick(e);
    }
  };

  TypeTester.prototype.handleOnChange = function handleOnChange(editorState) {
    var _this2 = this;

    var props = this.props;
    // const scale = props.scale
    var currentContent = editorState.getCurrentContent();
    // const selectionState = editorState.getSelection()
    // const currentBlock = currentContent.getBlockForKey(
    //   selectionState.getStartKey()
    // )

    this.setState({ editorState: editorState }, function () {
      // onChange callback lets you handle other interactions
      // outside of the type tester, based on the content
      // of the type tester.
      if (props.onChange && typeof props.onChange === 'function') {
        props.onChange(currentContent);
      }

      return _this2.handleAnyChange();
    });
  };

  TypeTester.prototype.handleAnyChange = function handleAnyChange(e) {
    var _this3 = this;

    var scale = this.props.scale;

    // TODO Could remove this and allow line breaks when scaling,
    // by pulling the font size out of the first block and applying
    // it to subsequent blocks
    if (typeof e !== 'undefined' && scale === 'once') {
      // Prevent default when we’re only scaling once
      return true;
    }

    if (window && scale !== false && this.props.scaleOnResize) {
      if (this.editorParentRef) {
        return this.resizeAll();
      } else {
        // Seem to need a delay for new content block to exist
        // TODO Figure out a way to…not need that. Might be the more
        // significant changes with fontSize stored in state
        // As of v0.13.x I’m checking for the ref first, and if it
        // already exists, we just go ahead and run resize all. If not,
        // use the old approach with a 1ms timeout
        this.timeoutresizeall = window.setTimeout(function () {
          return _this3.resizeAll();
        }, 1);
      }
    }

    return false; // Don’t prevent default
  };

  TypeTester.prototype.handleResize = function handleResize() {
    var _this4 = this;

    if (this.props.scaleOnResize) {
      this.setState({
        scaled: false
      }, function () {
        if (_this4.props.scale !== false) {
          // This works most of the time, but it’s actually just
          // working around resizeFonts needing to run until nothing
          // is line breaking anymore
          _this4.resizeAll({ breakWord: false });
          _this4.resizeAll({ breakWord: false });
          return;
        }
      });
    }
  };

  TypeTester.prototype.resizeAll = function resizeAll(opts) {
    var _Object$assign,
        _this5 = this;

    var scale = this.props.scale;
    var scaled = this.state.scaled;

    if (scale === 'once' && scaled === true) {
      return false;
    }

    opts = opts || {};
    opts = Object.assign(opts, (_Object$assign = {
      animate: true,
      breakWord: scale === 'once' }, _Object$assign['animate'] = false, _Object$assign));

    var container = this.editorParentRef;
    var lines = container.querySelectorAll('.js-line');
    var namespace = this.props.classNameNamespace;

    Array.prototype.forEach.call(lines, function (line, i) {
      _this5.resizeFonts(line, opts, function (el) {
        if (opts.animate) {
          el.classList.add(namespace + 'fadein');
          el.classList.remove(namespace + 'invisible');
        }
      });
    });

    if (!scaled) {
      this.setState({ scaled: true });
    }
  };

  TypeTester.prototype.getComputedStyle = function getComputedStyle(el) {
    if (window) {
      return window.getComputedStyle(el);
    }
  };

  TypeTester.prototype.getComputedDimensions = function getComputedDimensions(el) {
    var elStyle = this.getComputedStyle(el);
    return {
      width: parseInt(elStyle.getPropertyValue('width'), 10),
      height: parseInt(elStyle.getPropertyValue('height'), 10)
    };
  };

  // `testLineDimensions` and `dimensions` is based on github.com/zachleat/bigtext and my
  // jQuery-free fork, github.com/kennethormandy/big-ideas-text
  // https://git.io/JmX9k
  // Used under the MIT license: https://git.io/JmX9l


  TypeTester.prototype.testLineDimensions = function testLineDimensions(line, maxWidth, size, interval) {
    var currentStyle = this.getComputedStyle(line);
    var width = void 0;
    var property = 'font-size';
    // previousWidth = typeof previousWidth === 'number' ? previousWidth : 0
    line.style[property] = size + 'px';

    width = parseFloat(currentStyle.getPropertyValue('width').split('px')[0], 10);

    if (width >= maxWidth) {
      line.style[property] = '';
      if (width === maxWidth) {
        return {
          match: 'exact',
          size: parseFloat((parseFloat(size) - 0.1).toFixed(6))
        };
      }

      // Not using this `word-spacing` supprot from the original:
      // Since this is an estimate, we calculate how far over the width we went with the new value.
      // If this is word-spacing (our last resort guess) and the over is less than the under, we keep the higher value.
      // Otherwise, we revert to the underestimate.
      // var under = maxWidth - previousWidth
      // var over = width - maxWidth

      return {
        match: 'estimate',
        size: parseFloat((parseFloat(size) - interval).toFixed(3))
      };
    }

    return width;
  };

  TypeTester.prototype.dimensions = function dimensions(line, maxWidth) {
    // TODO replace 8, 4 with a proportional size to the calculated font-size.
    // var intervals = this.props.wholeNumberFontSizeOnly === false ? [8, 4, 1] : [8, 4, 1, 0.1]
    var intervals = [8, 4, 1, 0.1];
    var currentStyle = this.getComputedStyle(line);
    var width = parseInt(currentStyle.getPropertyValue('width'), 10);
    var currentFontSize = parseFloat(currentStyle.getPropertyValue('font-size'), 10);
    var newFontSize = 0;

    // TODO we can cache this ratio?
    var autoGuessSubtraction = 32; // font size in px
    var ratio = (width / currentFontSize).toFixed(6);
    newFontSize = parseInt(maxWidth / ratio, 10) - autoGuessSubtraction;

    outer: for (var m = 0, n = intervals.length; m < n; m++) {
      inner: for (var j = 1, k = 10; j <= k; j++) {
        if (newFontSize + j * intervals[m] > this.props.maxFontSize) {
          newFontSize = this.props.maxFontSize;
          break outer;
        }

        var lineMax = this.testLineDimensions(line, maxWidth, newFontSize + j * intervals[m], intervals[m]
        // lineMax
        );

        if (typeof lineMax !== 'number') {
          newFontSize = lineMax.size;
          if (lineMax.match === 'exact') {
            break outer;
          }
          break inner;
        }
      }
    }

    return newFontSize;
  };

  TypeTester.prototype.resizeFonts = function resizeFonts(el, opts, callback) {
    var d = this.getComputedDimensions(el.parentNode);
    var props = this.props;
    var minFontSize = props.minFontSize;
    var maxFontSize = props.maxFontSize;
    var fontSize = this.dimensions(el, d.width);
    var newFontSize = fontSize;

    if (fontSize < minFontSize) {
      el.style.fontSize = minFontSize + 'px';
      newFontSize = minFontSize;
      el.style.whiteSpace = 'normal';
      el.style.wordBreak = 'break-word';
    } else if (fontSize > maxFontSize) {
      el.style.fontSize = maxFontSize + 'px';
      newFontSize = maxFontSize;
      el.style.whiteSpace = 'nowrap';
    } else {
      el.style.fontSize = fontSize + 'px'; // State should handle this?
      if (opts.breakWord === true) {
        el.style.whiteSpace = 'normal';
        el.style.wordBreak = 'break-all'; // For Firefox, #16
      } else {
        el.style.whiteSpace = 'nowrap';
      }

      el.style.wordBreak = 'break-word';
    }

    if (props.onChangeFontSize && typeof props.onChangeFontSize === 'function') {
      props.onChangeFontSize(newFontSize);
    }

    if (typeof callback === 'function') {
      callback(el);
    }

    // this.state.fontSize = fontSize // …like this, but, you know, something that works.
  };

  TypeTester.prototype.resetState = function resetState(val) {
    if (typeof val === 'undefined') {
      return {};
    }

    return {
      editorState: EditorState.createWithContent(ContentState.createFromText(val)),
      valueOriginal: val,
      valueLabel: val.substr(0, 30),
      fontsLoaded: this.state.fontsLoaded,
      scaled: false
    };
  };

  TypeTester.prototype.componentWillUnmount = function componentWillUnmount() {
    window.clearTimeout(this.timeoutReRun);
    window.clearTimeout(this.timeoutResizeAll);

    window.removeEventListener('resize', this.handleResize, false);
  };

  TypeTester.prototype.componentFontLoaded = function componentFontLoaded() {
    var _this6 = this;

    this.setState({ fontsLoaded: true }, function () {
      _this6.resizeAll();
      window.addEventListener('resize', _this6.handleResize, false);
    });
  };

  // Check to see if value has changed from originally set value,
  // ex. for cycling through different defaults. Could choose
  // stop the cycle once the state has been edited once.
  // Also added support for scale prop, if you change from
  // scaling to now. Should actually be a list of props
  // that would trigger a prop reset


  TypeTester.prototype.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {
    var _this7 = this;

    var val = nextProps.value;
    var scale = nextProps.scale;

    // TODO Found some issue here when props.scale was stored
    // in parent state (ex. it was possible to toggle it)
    console.log(scale, this.state.scaled);

    // Might be solved by removing use of
    // componentWillReceiveProps anyway
    if (typeof val !== 'undefined' && (val !== this.state.valueOriginal || scale !== this.state.scaled)) {
      this.setState(this.resetState(val), function () {
        _this7.initFontCheck();
      });
    }
  };

  TypeTester.prototype.componentDidMount = function componentDidMount() {
    var _this8 = this;

    this.initFontCheck();

    /*
    Hack for Safari v10.0 and lower to re-run the numbers because it
    seems to get it wrong the first time, but then it works
    once you click or resize. There is probably another way
    around this, but it will be replacable by fixing the
    approach with TypeLine and the fontSize state too
      This issue also interfering with scale="once" support in Safari
      Possible is only using 50% of the width?
      Actually this might have something to do with Promises or other
      ES6 things, because most of it was added in Safari v10.1
    */
    this.timeoutReRun = window.setTimeout(function () {
      window.clearTimeout(_this8.timeoutReRun);
      if (_this8.state.fontsLoaded === true) {
        _this8.initFontCheck();
      }
    }, 500);
  };

  TypeTester.prototype.initFontCheck = function initFontCheck() {
    var _this9 = this;

    var el = this.editorParentRef;
    var props = this.props;
    var state = this.state;
    var computedStyle = window.getComputedStyle(el, null);
    var fontFamilies = computedStyle.getPropertyValue('font-family');
    var fontFamily = fontFamilies.split(',')[0].replace(/['"]+/g, '').trim();
    // const fontStretch = computedStyle.getPropertyValue('font-stretch')
    var scale = props.scale;
    var fontWeight = computedStyle.getPropertyValue('font-weight').toString();

    // This could be its own little one-function library
    // TODO Should have a test using the keywords
    // https://developer.mozilla.org/en-US/docs/Web/CSS/font-weight#Values
    if (fontWeight === 'normal') {
      fontWeight = 400;
    } else if (fontWeight === 'bold') {
      fontWeight = 700;
    }

    var observer = new FontFaceObserver(fontFamily, { weight: fontWeight });

    if (scale) {
      if (state.fontsLoaded === true) {
        this.componentFontLoaded();
      } else {
        // TODO Consider changing these props to a single
        // fontFaceObserver config object
        observer.load(props.testString, props.timeout).then(function () {
          _this9.componentFontLoaded();
        });
      }
    }
  };

  TypeTester.prototype.myBlockRenderer = function myBlockRenderer(contentBlock) {
    var state = this.state;
    var props = this.props;
    var type = contentBlock.getType();
    var scale = props.scale;

    if (type === 'unstyled' && (scale === true || scale === 'once')) {
      return {
        component: TypeLine,
        editable: !props.readOnly,
        props: {
          classNameNamespace: props.classNameNamespace,
          scale: scale,
          fontsLoaded: state.fontsLoaded,
          textAlignment: props.textAlignment,
          forceLineHeight: props.forceLineHeight
          // ,
          // onClick: () => {
          // console.log('click')
          // }
        }
      };
    }
  };

  TypeTester.prototype.render = function render() {
    var props = this.props;
    var state = this.state;
    var namespace = props.classNameNamespace;
    var styles = {};

    if (props.overflowX || props.overflowY) {
      styles.overflowX = props.overflowX ? 'hidden' : 'visible';
      styles.overflowY = props.overflowY ? 'hidden' : 'visible';
    }

    var classes = [namespace.slice(0, -1), props.className, state.caret ? namespace + 'caret' : ''].join(' ');

    var remainingProps = {
      readOnly: props.readOnly,
      placeholder: props.placeholder,
      textAlignment: props.textAlignment,
      editorKey: props.editorKey
    };

    return React.createElement(
      'div',
      {
        style: styles,
        className: classes,
        onClick: this.handleOnClickParent,
        ref: this.handleEditorParentRef },
      React.createElement(Editor, _extends({
        ref: this.handleEditorRef,
        editorState: state.editorState,
        onChange: this.handleOnChange,
        onFocus: this.handleOnFocus,
        handleReturn: this.handleAnyChange,
        stripPastedStyles: true,
        spellCheck: false,
        blockRendererFn: this.myBlockRenderer.bind(this)
      }, remainingProps))
    );
  };

  return TypeTester;
}(Component);

TypeTester.defaultProps = {
  readOnly: false,
  className: '',
  placeholder: 'Edit…',
  value: '',
  testString: 'null', // null uses default fontfaceobserver test string
  textAlignment: 'left',
  scale: false,
  scaleOnResize: true,
  minFontSize: 16,
  maxFontSize: 500,
  forceLineHeight: false,
  caret: false,
  overflowX: false,
  overflowY: false,
  timeout: 10000,
  classNameNamespace: 'public-TypeTester-'
};

TypeTester.propTypes = process.env.NODE_ENV !== "production" ? {
  readOnly: PropTypes.bool,
  className: PropTypes.string,
  placeholder: PropTypes.string.isRequired,
  value: PropTypes.string.isRequired,
  testString: PropTypes.string,
  textAlignment: PropTypes.string,
  scale: PropTypes.oneOf([true, false, 'once']),
  scaleOnResize: PropTypes.bool,
  minFontSize: PropTypes.number,
  maxFontSize: PropTypes.number,
  forceLineHeight: PropTypes.bool,
  caret: PropTypes.bool,
  overflowX: PropTypes.bool,
  overflowY: PropTypes.bool,
  timeout: PropTypes.number,
  classNameNamespace: PropTypes.string,
  onClick: PropTypes.func,
  onChange: PropTypes.func,
  onChangeFontSize: PropTypes.func,
  editorKey: PropTypes.string,
  analytics: function analytics(props, propName, componentName) {
    if (props[propName] !== undefined) {
      return new Error('Invalid prop `' + propName + '` supplied to `' + componentName + '`. Replace by passing a function to `onClick`.');
    }
  }
} : {};

export default TypeTester;